// --------------------------------------------------
// CONSOLE FOR HATP (MODULE)
// --------------------------------------------------

#include <iostream>

#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>

#include <QApplication>

#include "MsgThread.hh"
#include "MainWindow.hh"

#define DEFAULT_ADDRESS "localhost"
#define DEFAULT_PORT "5500"

int main(int argc, char ** argv){
	std::string address (DEFAULT_ADDRESS);
	std::string port (DEFAULT_PORT);
	
	//Supported options
	boost::program_options::options_description desc("Allowed options");

	desc.add_options()
		("help,h", "Produce help message")
		("address,a", boost::program_options::value<std::string>(&address), "The address of the server (default : localhost)")
		("port,p", boost::program_options::value<std::string>(&port), "The port number of the server (default : 5500)")
		("image,i", "Generate the jpeg images of both tree plan and stream plan (in addition to the already generated svg vectorial images)");
	
	//The order for parameter (without the -.. options) is : address port
	boost::program_options::positional_options_description positional;
	positional.add("address", 1);
	positional.add("port", 1);

	//Parse the parameters
	boost::program_options::variables_map variables;
	try{
		boost::program_options::store(boost::program_options::command_line_parser(argc, argv).options(desc).positional(positional).run(), variables);
	}catch(boost::program_options::validation_error e){
		//Called if a parameter get a value that does not match its type
		std::cerr<<"ERROR: Catch a validation error exception : "<<e.what()<<std::endl;
		exit(99);
	}catch(boost::program_options::unknown_option e){
		//Called if an unknown option is given (e.g. : -z)
		std::cerr<<"WARNING: Catch an excpetion : "<<e.what()<<std::endl;
	}
	boost::program_options::notify(variables);
	
	//Print usage
	if(variables.count("help")){
		std::string software(argv[0]);
		std::cout<<"Usage:\tDefault:\t"<<software<<std::endl;
		std::cout<<"\tCompact:\t"<<software<<" address:port"<<std::endl;
		std::cout<<"\tStandard:\t"<<software<<" address port"<<std::endl;
		std::cout<<"\tComplete:\t"<<software<<" [-a address] [-p port]"<<std::endl<<std::endl;
		std::cout<<desc<<std::endl;
		return 0;
	}
	
	//Allow to write "address:port" (in addition to "address port")
	if(address.find(":")!=std::string::npos){
		//The address contains the port number as well
		size_t pos=address.find(":");
		
		std::string tempPort=address.substr(pos+1);
		if(!tempPort.empty()){
			unsigned int testPort; //To test if the port is valide
			try{
				testPort=boost::lexical_cast<unsigned int>(tempPort);
			}catch(const boost::bad_lexical_cast &){
				std::cerr<<"ERROR: The port must be a number !"<<std::endl;
				exit(99);
			}
			try{
				port=boost::lexical_cast<std::string>(testPort);
			}catch(const boost::bad_lexical_cast &){
				std::cerr<<"ERROR: The port must be a number !"<<std::endl;
				exit(99);
			}
		}
		
		if(pos!=0){
			address=address.substr(0, pos);
		}else{
			address="localhost";
		}
	}
	
	bool automaticSave(variables.count("image"));
	
	int fakeArgc=1; //To change if one want to use the possibilities of Qt
	char *fakeArgv[1]={argv[0]};
	QApplication app(fakeArgc, fakeArgv);

	MsgThread *thread = new MsgThread();
	thread->startListening(address, port);

	if(!thread->testConnection()) {
		std::cerr << "Unable to connect to MsgConnector Server... exited." << std::endl;
		return(1);
	}

	std::cout<<"Starts "<<argv[0]<<" and connect to "<<address<<":"<<port<<std::endl;

	MainWindow window(thread);
	window.setAutomaticImgSave(automaticSave);
	window.setApplicationTitle("HATP Console (VIEWER)");

	window.addSystemMessage("Waiting for connection...");
	window.setStatusMessage("Waiting for connection...");

	window.show();

	return app.exec();
}
